﻿// Switch to toggle between LightningCharts v6.5.8 and v7.0.18/v7.1.1
#define ARCTION_V7

using System;
using System.Diagnostics;
using System.Drawing;
using System.IO;
using System.Windows.Forms;
#if ARCTION_V7
using Arction.WinForms.Charting;
using Arction.WinForms.Charting.SeriesXY;
using Arction.WinForms.Charting.Views.ViewXY;
#else
using Arction.LightningChartUltimate;
using Arction.LightningChartUltimate.SeriesXY;
using Arction.LightningChartUltimate.Views.ViewXY;
#endif

namespace AL_IntensityGrid_Logarithmic
{
    public partial class Form1 : Form
    {
        //internal static string LicenseKey = "...";

        const bool LOGARITHMIC = true;

        #region Member-variables

        int COLUMNS = 20;
        int ROWS = 20;
        double VIEW_X_MIN = 0;
        double VIEW_X_MAX = 65;
        double VIEW_Y_MIN = 0;
        double VIEW_Y_MAX = 10000;
        private double _valueMin = 0.0;
        private double _valueMax = 1.0;

        private double[][] _data;

        private LightningChartUltimate _lcu;
        ViewXY _view;
        private IntensityGridSeries _series;

        #endregion

        #region Constructor

        public Form1()
        {
            InitializeComponent();

            GenerateData();
            CreateChart();
            SetData(_data, COLUMNS, ROWS, VIEW_X_MIN, VIEW_X_MAX, VIEW_Y_MIN, VIEW_Y_MAX);
        }

        #endregion

        #region GenerateData()

        private void GenerateData()
        {
            var random = new Random();
            _data = new double[COLUMNS][];
            for (int x = 0; x < COLUMNS; x++)
            {
                var colData = new double[ROWS];
                for (int y = 0; y < ROWS; y++)
                {
                    var cellValue = 40 * ((double)x / COLUMNS) + 10 * ((double)y / ROWS) + 28.0 * random.NextDouble();
                    colData[y] = cellValue;
                }
                _data[x] = colData;
            }
        }

        #endregion

        #region LCU-Wrapper

        void SafeEndUpdate()
        {
            try
            {
                _lcu.EndUpdate();
            }
            catch (Exception ex)
            {
                Debug.WriteLine(ex.Message);
            }
        }

        #endregion

        #region CreateChart()

        private void CreateChart()
        {
            _lcu = new LightningChartUltimate();

            _lcu.Dock = DockStyle.Fill;

            _lcu.BeginUpdate();
            //_lcu.LicenseKey = LicenseKey;

#if ARCTION_V7
            _lcu.ThrowChartExceptions = true;
            _lcu.ChartError += _lcu_ChartError;
#endif

            _lcu.ActiveView = ActiveView.ViewXY;
            _lcu.Title.Visible = false;

            _view = _lcu.ViewXY;
            _view.LegendBox.Visible = false;
            _view.YAxes[0].ScaleType = LOGARITHMIC ? ScaleType.Logarithmic : ScaleType.Linear;

            #region CreateSeries()

            _series = new IntensityGridSeries(_view, _view.XAxes[0], _view.YAxes[0]);
            _series.MouseInteraction = false;
            _series.PixelRendering = !LOGARITHMIC;
            _series.Optimization = IntensitySeriesOptimization.DynamicData;

            _view.IntensityGridSeries.Add(_series);

            #endregion

            SafeEndUpdate();

            this.Controls.Add(_lcu);
        }

        #endregion
        #region SetData

        public void SetData(double[][] data, int resX, int resY, double minX, double maxX, double minY, double maxY)
        {
            try
            {
                _lcu.BeginUpdate();
                _series.SetRangesXY(minX, maxX, minY, maxY);

                var data2 = new IntensityPoint[resX, resY];
                for (int x = 0; x < resX; x++)
                    for (int y = 0; y < resY; y++)
                    {
                        var value = data[x][y];
                        if (double.IsNaN(value))
                            value = -1;
                        else if (value < 0)
                            value = -2;
                        data2[x, y] = new IntensityPoint() { Value = value };
                    }

                _series.Data = data2;

                _view.ZoomToFit();
            }
            catch (Exception ex)
            {
                Debug.WriteLine(ex.Message);
            }
            SafeEndUpdate();
        }

        #endregion


        void _lcu_ChartError(ErrorType errorType, string errorDescription, string details, LightningChartUltimate chart, object source, DateTime timeStamp)
        {
            Debug.WriteLine("Arction Exception: {0}, {1}, {2}", errorType, errorDescription, details);
        }
    }
}
